precision highp float;

uniform vec3 scaler; 
uniform vec3 transer;
uniform float pitch;
uniform float yaw;
uniform float roll;
uniform float eye;
uniform mat4 quat;
attribute vec3 vPosition;
attribute vec2 vTex_coord;
varying vec3 tex_coord;


mat4 rotationMatrix(vec3 axis, float angle) {
 axis = normalize(axis);
 float s = sin(angle);
 float c = cos(angle);
 float oc = 1.0 - c;
    
 return mat4(
  oc * axis.x * axis.x + c,          
  oc * axis.x * axis.y - axis.z * s,  
  oc * axis.z * axis.x + axis.y * s, 
  0.0,
  oc * axis.x * axis.y + axis.z * s, 
  oc * axis.y * axis.y + c,    
  oc * axis.y * axis.z - axis.x * s,  
  0.0,
  oc * axis.z * axis.x - axis.y * s,  
  oc * axis.y * axis.z + axis.x * s,
  oc * axis.z * axis.z + c,    
  0.0, 
  0.0, 
  0.0, 
  0.0,
  1.0);
}


mat4 per_matrix(float fov, float aspect, float near, float far) {
 float D2R = 3.141592 / 180.0;
 float yScale = 1.0/ tan(D2R * fov /2.0);
 float xScale = yScale / aspect;
 float nearmfar = near - far;
 return mat4(
  xScale, 0.0, 0.0, 0.0, 
  0.0, yScale, 0.0, 0.0,
  0.0, 0.0, (far + near) / nearmfar, -1.0,
  0.0, 0.0, 2.0*far*near / nearmfar, 0.0);
  
}

mat4 quat_to_matrix(vec4 q) {
 mat4 a,b;
 a =  mat4( q.w, q.z, -q.y, q.x,
           -q.z, q.w, q.x, q.y,
            q.y, -q.x, q.w, q.z,
           -q.x,-q.y,-q.z, q.w);
 b = mat4( q.w, q.z, -q.y, -q.x,
          -q.z, q.w, q.x,-q.y,
           q.y,-q.x, q.w,-q.z,
           q.x, q.y, q.z, q.w);
 return a * b;

}


void main() {
 vec4 pos;
 mat4 matrix1, matrix2, matrix3;


 pos = vec4(vPosition - transer, -1.0 * (vPosition - transer).z);

 matrix1 = rotationMatrix(vec3(1.0, 0.0, 0.0), pitch);
 matrix2 = rotationMatrix(vec3(0.0, 1.0, 0.0), yaw);
 matrix3 = rotationMatrix(vec3(0.0, 0.0, 1.0), roll);

 pos*= (matrix1 * matrix2 * matrix3);
// pos *= quat;

// pos *= quat_to_matrix(quat);
 
 pos.xyz *= scaler;

 pos.x += eye;
 
// pos.z /= 100.0f;
 gl_Position = vec4(pos.xyz, -1.0 *pos.z); 
 tex_coord = vec3(vTex_coord, pos.z * 1.0); 
} 

